__all__ = [
    "CONFIG_PATH_ENV_NAME",
    "CORS_ORIGINS_ENV_NAME",
    "ROOT_PATH_ENV_NAME",
]

CONFIG_PATH_ENV_NAME = "QUALIBRATE_COMPOSITE_CONFIG_FILE"
ROOT_PATH_ENV_NAME = "QUALIBRATE_COMPOSITE_ROOT_PATH"
CORS_ORIGINS_ENV_NAME = "QUALIBRATE_COMPOSITE_CORS_ORIGINS"
